* Calculate expected percent correctly predicted, expected percent in modal
* category, and expected percent reduction in error for probit, logit, and
* related models.  Also optionally presents a classification table (cfr lstat).
*
* Implements algorithm suggested by Herron (2000) with extensions suggested
* by Harvey Palmer for polychotomous probit and logit and for an estimated
* "expected proportional reduction in error" score.
*
* Note that the "ePMC" presented here is mathematically the same as the
* "proportional chance criterion" presented in other works, apparently
* derived in Morrison (1969) and presented in Hair et al. (1998).
*
* Fixed to work with multinomial logit - thanks to Raymond Hicks.
* April 2013

version 7.0
capture program drop epcp
program define epcp
        local binary = inlist("`e(cmd)'", "logit", "probit", "logistic", "cloglog")
        if (`binary' == 0 & inlist("`e(cmd)'", "oprobit", "ologit") == 0 &/*
         */ inlist("`e(cmd)'", "mlogit", "nlogit") == 0) {
                error 301
        }
        syntax, [notable cutoff(real 0.5) predict(string)]
        quietly count
        local obs = r(N)
        if "`predict'" ~= "" {
            local pcat = "`predict'"
        }
        else {
            tempvar pcat
        }
        quietly gen byte `pcat' = .
        tempvar insamp depvar
        quietly gen byte `insamp' = e(sample)
        * This has to be done for the binary case
        quietly gen byte `depvar' = `e(depvar)'
        label var `pcat' "Predicted values of `e(depvar)'"
        local vlabel : value label `e(depvar)'
        label var `depvar' "Actual values of `e(depvar)'"
        capture label val `pcat' `vlabel'
        capture label val `depvar' `vlabel'
        local pcp = 0
        local epcp = 0
        if `binary' == 1 {
            tempvar pred
            quietly predict `pred'
            quietly replace `depvar' = (`e(depvar)' ~= 0)
            local cat0 = 0
            local cat1 = 0
            forvalues i = 1/`obs' {
                if `insamp'[`i'] == 0 {
                        continue
                }
                local cat0 = `cat0' + (1-`depvar'[`i'])
                local cat1 = `cat1' + `depvar'[`i']
                quietly replace `pcat' = `pred'[`i']>= `cutoff' in `i'
                local pcp = `pcp' + (`depvar'[`i'] == `pcat'[`i'])
                local epcp = `epcp' + `pred'[`i']*`depvar'[`i']
                local epcp = `epcp' + (1-`pred'[`i'])*(1-`depvar'[`i'])
            }
            if `cat0' > `cat1' {
                local pmc = `cat0'/`e(N)'
            }
            else {
                local pmc = `cat1'/`e(N)'
            }
            local epmc = ((`cat0'*`cat0')+(`cat1'*`cat1'))/(`e(N)'*`e(N)')
        }
        else {
            if inlist("`e(cmd)'", "oprobit", "ologit") {
                local cats = `e(k_cat)'
            }
            if "`e(cmd)'"=="mlogit" {
                local cats = `e(k_out)'
            }
            local preds = ""
            local vals = ""
            forvalues i = 1/`cats' {
                tempvar pred`i'
                local cat`i' = 0
                local preds = "`preds'" + "`pred`i'' "
                if inlist("`e(cmd)'", "oprobit", "ologit") {
                    local vals = "`vals'" + string(el(e(cat), 1, `i')) +" "
                }
                if "`e(cmd)'"=="mlogit" {
                    local vals = "`vals'" + string(el(e(out), 1, `i')) +" "
                }
            }
            quietly predict `preds'
            forvalues i = 1/`obs' {
                if `insamp'[`i'] == 0 {
                    continue
                }
                local maxcat = 0
                local maxpred = -1
                forvalues j = 1/`cats' {
                    local value : word `j' of `vals'
                    local cat`j' = `cat`j'' + (`depvar'[`i'] == `value')
                    if `depvar'[`i'] == `value' {
                        local epcp = `epcp' + `pred`j''[`i']
                    }
                    if `maxpred' < `pred`j''[`i'] {
                        local maxpred = `pred`j''[`i']
                        local maxval = `value'
                    }
                }
                local pcp = `pcp' + (`depvar'[`i'] == `maxval')
                quietly replace `pcat' = `maxval' in `i'
            }
            local maxcat = 0
            local maxcount = -1
            local epmc = 0
            forvalues i = 1/`cats' {
                if `maxcount' < `cat`i'' {
                   local maxcat = `i'
                   local maxcount = `cat`i''
                }
                local epmc = `epmc' + `cat`i''*`cat`i''
            }
            local pmc = `maxcount' / `e(N)'
            local epmc = `epmc'/(`e(N)'*`e(N)')
        }
        local pcp = (`pcp' / `e(N)')*100
        local epcp = (`epcp' / `e(N)')*100
        local pmc = `pmc'*100
        local epmc = `epmc'*100
        local pre = ((`pcp'-`pmc')/(100-`pmc'))*100
        local epre = ((`epcp'-`epmc')/(100-`epmc'))*100
        di as text
        di as text "Displaying classification results for `e(cmd)'"
        di as text "Dependent variable: `e(depvar)'"
        if `binary' == 1 {
           di as text "Using cutoff score of `cutoff'"
        }
        di as text
        if "`table'" ~= "notable" {
           di as text "Classification table"
           tabulate `pcat' `depvar'
        }
        di as text "    Percent Correctly Predicted = " as res %6.2f `pcp' "%"
        di as text "      Percent in Modal Category = " as res %6.2f `pmc' "%"
        di as text "Proportional Reduction in Error = " as res %6.2f `pre' "%"
        di as text
        di as text "expected PCP = " as result %6.2f `epcp' "%"
        di as text "expected PMC = " as result %6.2f `epmc' "%"
        di as text "expected PRE = " as result %6.2f `epre' "%"
end
